#!/usr/bin/python3
'''
=======================================================================
Scope = 'Read in data radial data files and average consecutive gates.\n'+\
        'compresses to byte format for udp packets and write out to\n'+\
        'file for sending.'

  This code is owned by Remote Sensing Solutions. It is intended for use
  by Dr. Paul Chang of NOAA/NESDIS to provide NCO with real-time radial data
  from the NOAA WP-3D Tail Doppler Radar. It is to be deployed on the NOAA
  Aircraft Operations server. Use beyond the above purpose or distribution
  beyond AOC requires approval from Dr. James Carswell of Remote Sensing
  Solutions, 3179 Main Street, Barnstable, MA 02630.

Written by: \
Author = 'James Carswell and Dan Robinson'
Institution = 'Remote Sensing Solutions, Inc.'

Version
versionTag = '0.1.1'

CONTRIBUTORS:  Sonia Otero, OMAO/MACC

 Modifications
  (List version, author, date and then changes.)

  V0.1.1  - Jim Carswell, 20100208
  1) Added check when calculating numberOfAveBins to use the smaller of
     numberOfBins (in header) and maxNumberOfGates.

 2022.4.18: SO - Converted to Python 3.6. Eliminated references to the
        variable type 'long'; use str.encode() to write as byte array.
        Eliminated the unnecessary step of creating zip files.
=======================================================================
'''
# Import Modules
import time, string, sys, struct, array, os.path


def openNewFile(path, flid, TDNumber, myTime):
    fileName = '%10s%d-%s.dat' % \
               (flid,TDNumber, time.strftime('%H%M', myTime))
    print('\nCreating file', fileName)
    outputFile = open(os.path.join(path, fileName),'wb')
    return outputFile, fileName


def finalizeFile( fileObject, filename, totalLength, offset ):
    fileObject.flush()
    fileObject.seek( offset )
    strLength = '%010d' % (totalLength)   
    fileObject.write( strLength.encode() )
    # move pointer to EOF and close
    fileObject.seek( 0,2 )
    print('\nFile complete. Closing file',filename)
    fileObject.close( )


def processRadialFile(    radialFilePath, \
                          outputFilePath, \
                          numberOfGatesToAverage, \
                          maxNumberOfGates, \
                          flid, \
                          verbose):
    headerTag = 0
    dataTag   = 1
    stdOutCnt = " "

    radialFile = open(radialFilePath,mode='r')

    # Read Input File Header
    header=''
    for lineNumber in range(8):
        header=header+radialFile.readline()
    splitHeader = header.split('\n')
    splitHeader = splitHeader[0:8]
    if verbose==1:
        print("splitHeader:",splitHeader)
    
    # Build Output Header Structure
    missionId           = int(splitHeader[0])
    aircraftId          = int(splitHeader[1])
    TDNumber            = int(splitHeader[2])
    numberOfBins        = int(splitHeader[3])
    numberOfAveBins=0
    if numberOfBins <= maxNumberOfGates:
        numberOfAveBins = int(numberOfBins/numberOfGatesToAverage)
    else:
        numberOfAveBins = int(int(maxNumberOfGates)/numberOfGatesToAverage)
    rangeDelay          = float(splitHeader[4])
    rangeGateResolution = float(splitHeader[5])*numberOfGatesToAverage
    reflectivityFlag    = int(splitHeader[6][0])
    DopplerFlag         = int(splitHeader[6][1])
    spectralWidthFlag   = int(splitHeader[6][2])
    dataType            = reflectivityFlag + DopplerFlag*2 + \
                           spectralWidthFlag*4
    antennaNumber       = int(splitHeader[7])

    if verbose==1:
        print("missionId:",missionId)
        print("aircraftId:",aircraftId)
        print("TDNumber:",TDNumber)
        print("numberOfBins:",numberOfBins)
        print("numberOfAveBins:",numberOfAveBins)
        print("rangeDelay:",rangeDelay)
        print("rangeGateResolution:",rangeGateResolution)
        print("reflectivityFlag:",reflectivityFlag)
        print("DopplerFlag:",DopplerFlag)
        print("spectralWidthFlag:",spectralWidthFlag)
        print("antennaNumber:",antennaNumber)

    headerFormat = '=i3h2f2h'
    packedHeader = struct.pack(headerFormat, missionId, aircraftId, TDNumber, numberOfAveBins, \
                               rangeDelay, rangeGateResolution, dataType, antennaNumber )
    
    # Determine data block read size in bytes
    numberOfParameters = reflectivityFlag + DopplerFlag + spectralWidthFlag
    numberOfBinsPerLine      = 10
    numberOfCharactersPerBin = 6
    numberOfLineFeeds     = int(numberOfBins/numberOfBinsPerLine)
    if (numberOfBins % numberOfBinsPerLine) != 0:
        numberOfLineFeeds = numberOfLineFeeds+1
    numberOfBytesPerBlock = numberOfBins*numberOfCharactersPerBin + \
                            numberOfLineFeeds
    
    eof = 0
    blockCount = int(-1)
    headerBlockCount = 0
    rHeader = radialFile.readline()
    if verbose==1:
        print("rHeader:",rHeader)

    bytesWritten = 0
    lastTime = time.time()
    startTime = lastTime
    z = 0
    outputBuffer = array.array('B') 
    tlvStructHeader = '=2HL'
    tlvStructHeaderSize = struct.calcsize(tlvStructHeader)
    print("tlvStructHeaderSize:",tlvStructHeaderSize)
    minPayload = 1500
    maxPayload = 0
    minCount = 0
    maxCount = 0
    payloadLen = 0
    readCount = 0
    firstPointTime = 0.0
    newFileTime = 300.0  # 5 minutes
    fileCreated = 0
    bulletinPrefix = '****0000000000****\n'
    prefixLengthOffset = 4 # start of length field in prefix
    fileCount = 0
    compFileStart = 0

    # Loop through the file
    while rHeader:

        radarHeaderFormat = '=3hd4f'
        radarHeaderLen    = struct.calcsize(radarHeaderFormat)
        
        # Read in data block
        radarHeader       = rHeader.split()
        unixTime          = float(radarHeader[0])
        latitude          = float(radarHeader[1])
        longitude         = float(radarHeader[2])
        altitude          = int(float(radarHeader[3]))
        azimuth           = float(radarHeader[4])
        incidence         = float(radarHeader[5])

        # Read in radar data block
        readBlock = radialFile.read(numberOfBytesPerBlock)
        if (readBlock):
            dataBlock = readBlock.split()

            # Process Doppler Data
            if DopplerFlag:
                Doppler = array.array('f')
                if len(dataBlock) < numberOfBins+1: 
                    
                    # Convert to float array
                    n = len(dataBlock)
                    if n > maxNumberOfGates: n = maxNumberOfGates
                    for bin in range(n):
                        Doppler = Doppler + \
                            array.array('f',[float(dataBlock[bin].strip('\n'))])
                    
                    # Average data and create output array (short, tenths of m/s)
                    aveDoppler = array.array('h')
                    for aveBin in range(numberOfAveBins):
                        tmp=0.
                        n=0.
                        for i in range(numberOfGatesToAverage):
                            bin = (aveBin*numberOfGatesToAverage)+i
                            if abs(Doppler[bin]) < 4096:
                                n=n+1
                                tmp = tmp+Doppler[bin]
                        if n > 0: 
                            tmp1 = int((tmp/n+4096.)+.5)
                            aveDoppler = aveDoppler + \
                                array.array('h', [int(tmp1-4096.)])
                        else:
                            aveDoppler = aveDoppler + \
                                array.array('h',[-8888])
                            
                    # Write radar header
                    radarHeaderOut = struct.pack('=dfffff', unixTime, latitude, longitude, altitude,\
                                               azimuth, incidence)
                    
                    # Create new file?
                    timeDiff = unixTime - firstPointTime
                    if timeDiff > newFileTime or fileCreated == 0:
                        if fileCreated == 1:
                            # include length of URNT16 header, but not prefix
                            totalLength = bytesWritten + len(urnt16Header)
                            finalizeFile( outputFile, fname, totalLength, prefixLengthOffset)

                        myTime = time.gmtime(unixTime)
                        outputFile, fname= openNewFile( outputFilePath, flid, TDNumber, myTime )
                        outputFile.write( bulletinPrefix.encode() )
                        # Append DateTimeGroup to URNT16 Header
                        # Use time of first data point
                        tupleTime = time.gmtime(unixTime)
                        urnt16Header = 'URNT16 KWBC ' + time.strftime('%d%H%M', tupleTime) + '\n'
                        outputFile.write( urnt16Header.encode() )
                        outputFile.write( packedHeader )
                        bytesWritten = len( packedHeader )
                        startTime = time.time()
                        firstPointTime = unixTime
                        firstDataPoint = 0
                        fileCreated = 1

                    bytesWritten += len(radarHeaderOut)+(len(aveDoppler)*aveDoppler.itemsize)
                    outputFile.write( radarHeaderOut )
                    outputFile.write( aveDoppler )
                    outputFile.flush()

                    # Block Counter 
                    if verbose ==1:
                        if blockCount % 10e3 == 0: 
                            thisTime = time.time()
                            print('Block Count:',blockCount, '  Processing Time: ',thisTime-lastTime)
                            lastTime = thisTime
                    else:
                        if blockCount % 1e3 == 0:
                            nbs = len(stdOutCnt)
                            stdOutCnt = 'Block Count: %s' % blockCount
                            sys.stdout.write('\b'*nbs+stdOutCnt)
                            sys.stdout.flush()
    
                else:
                    print('-> Bad Radar Block ',blockCount)
                    print(len(dataBlock), numberOfBins+1)
                
            # Read the radar header, if any
            rHeader = radialFile.readline()
    radialFile.close()
    print("\n Closing radial",radialFilePath)

    # Done looping through file, clean up currently open file and last archive
    if fileCreated == 1:
        # include length of URNT16 header, but not prefix
        totalLength = bytesWritten + len(urnt16Header)
        finalizeFile( outputFile, fname, totalLength, prefixLengthOffset)
