/* wind_functions.c
 *	Functions to convert among different representations of winds.
 */

/* pi/ 180 */
#ifndef DEG2RAD
# define DEG2RAD 0.0174533
#endif 

/* 180 / pi */	
#ifndef RAD2DEG
# define RAD2DEG 57.2957795	
#endif 
# include <math.h>
# include "wind_functions.h"
 
int wdir2uv(float wd, float ws, float *u, float *v, float bad)
/* given speed and direction in degrees meteorological ( direction 
 * from which wind is blowing ), return u, v  in same units as speed.
 */
{
	float ang;
	
	if( (wd < 0.0) || (wd > 360.0) ){
	        *u = bad ;
        	*v = bad ;
        	return -1;
	}
	else {
	        ang = -(90.0+wd)*DEG2RAD; /*  conversion to math angle */
        	*u = ws * cos(ang);
        	*v = ws * sin(ang);
        	return 0;
	} 
}

int uv2wd(float u, float v, float *wd, float *ws, float bad)
/* given u,v return speed in same units and direction in degrees, 
 * met sense (0 blowing from North, 90 deg blowing from 90 deg, etc...
 */
{
	float ang;
	
	if(u > bad){
		ang = atan2(v,u) * RAD2DEG;
	/* convert math angle to meteorological angle */
		*wd = fmod(360.0-(90.0+ang),360.);
		*ws = sqrt(u * u + v * v);
		return 0;
        }
	
	else {
		*wd = bad;
		*ws = bad;
		return -1;
	}
}

/* Following function converts wind direction and speed at x,y 
 * to tangential and  radial winds relative to a center of circulation
 * at xc, yc. xc_shft and ycshft are position corrections, used when
 * trying new centers of circulation, for example. Note use of pointers for
 * values that function will change. This is translated from Fortan...
 */
int wdws_to_vtvr(float wdir, float wspd, float x, float y, 
	float xc, float yc, int shifting, float xc_shft, float yc_shft,
	int rem_motion, float u_storm, float v_storm,
	float *u, float *v, float *vr, float *vt, float bad_val)
{
	double xrel, yrel, ang;

	if( wdir2uv(wdir, wspd, u, v, bad_val) != 0) { 
		*vr = bad_val;
		*vt = bad_val;
		return -1;
	}
	else {
		if(shifting){
			xrel = x - xc + xc_shft;
			yrel = y - yc + yc_shft;
		}
		else{
			xrel = x - xc;
			yrel = y - yc;
		}
		
		if(rem_motion){
			*u = *u - u_storm;
			*v = *v - v_storm;
		}
		if( (xrel == yrel) && (xrel == 0.0) ){  /* center */
			*vr = 0.0;
			*vt = 0.0;
		}
		else {
			ang = atan2(yrel, xrel);
			*vr = *u * cos(ang) + *v * sin(ang);
			*vt = -(*u) * sin(ang) + *v * cos(ang);
		}
	} 
	return 0;
}

int uv2VtVr(float u, float v, float phi, float *Vt, float *Vr)
/* calculate tangential and radial wind from u, v. Phi is the angle
 * of the point relative to the center that Vt and Vr defined for.
 * +Vr X +Vt points up.
 */
{
	*Vr = u * cos(phi) + v * sin(phi);
	*Vt = -u * sin(phi) + v * cos(phi);
	return 0;
}

int VtVr2uv(float Vt, float Vr, float phi, float *u, float *v)
/*  inverse of uv2VtVr */
{
	*v = Vt * cos(phi) + Vr * sin(phi);
	*u = -Vt * sin(phi) + Vr * cos(phi);
	return 0;
}

/* following function is C - version of erwind */
void my_erwind(float vr1, float vr2, float theta1, float theta2, 
	float phi1, float phi2, float w, float bad_val, float *u, float *v)
{
	float div, th1, th2, p1, p2;
/* equations for u,v if we assume w = 0 */
	th1 = theta1 * DEG2RAD;
	th2 = theta2 * DEG2RAD;
	p1 = phi1 * DEG2RAD;
	p2 = phi2 * DEG2RAD;
	
	if( vr1 > bad_val ) {
	   div = cos(th1)*cos(p1)*sin(th2)*cos(p2) -
		cos(th2)*cos(p2)*sin(th1)*cos(p1);
	   *u = (vr1 * sin(th2) * cos(p2) - vr2 * sin(th1) * cos(p1) ) /
		div;
	   *v = (vr2 * cos(th1) * cos(p1) - vr1 * cos(th2) * cos(p2) ) / 
		div;
	}
	else {
	   *u = bad_val;
	   *v = bad_val;
	}
	return;
}

