#!/usr/bin/python3
'''
=========================================================================
DESCRIPTION: Detects and processes Tail Doppler data files received
                from John Gamache's processor; and builds binary payload
                file that contains TLVs that should be sent to the ground
                server.

 ORIGINAL AUTHOR: Dan Robinson, RSS

 DATE: 2010.1.29

 Required applications:
     processRadialFile.py

 CONTRIBUTORS:  John Gamache, OAR/AOML/HRD
                Sonia Otero, OMAO/MACC

 REVISIONS:

 Modifications: (list version number, date, author, changes)
   V.0.1.1 - Jim Carswell, RSS
   1) Added verbose variable which also gets passed to parsing routine.
   2) Modified stdout.

   V2.0 - Dan Robinson, RSS, June 2012
   1) Monitors a directory on the radar workstation for new ASCII radar files
   2) Processes ASCII radar files into binary NCO formatted files
   3) Compresses a configurable number of binary files into an archive
   4) Standardized command line and config file formats


 2022.4.18: SO - Converted to Python 3.6 and new libraries (pathlib, asyncio).
                 Works in tandem with the repo AirGO, which provides utilities
                 to Python programs.

=========================================================================
'''
import os,sys,string,time,argparse
import asyncio,pathlib
sys.path.append('/home/sysop/hrdradar/scripts/')
from processRadialFiles import *

sys.path.insert(1,'/home/sysop/airgo/core')
import util as u

verbose = 0
radarConfigPath = pathlib.Path('/home/sysop/hrdradar/defaults/taradar.cfg')
sizeDir = {}
DELAY_MULTIPLE = 5
delayCount = 0

# ----------------------------------------------------------------------
# Procedures
# ----------------------------------------------------------------------

# radarsync puts EMC data land in INPUT_PATH
# For each *_radials.txt.gz whose growth stabilizes, move it to HARVEST_PATH
# ----------------------------------------------------------------------
async def scanIncomingPath():
    fileList = sorted(INPUT_PATH.glob('*_radials.txt.gz'))
    if not fileList or len(fileList)==0:
        global delayCount
        if (delayCount % DELAY_MULTIPLE) == 0:
            u.sendStatusToChat('aircraftTailRadarProcessor: No files in '+str(INPUT_PATH))
            delayCount = 0
        delayCount += 1
        return

    for f in fileList:
        size = u.moveFileIfMaxGrown(sizeDir, f, HARVEST_PATH)
        msg=' '.join(['aircraftTailRadarProcessor:',f.name,'has size',f'{size}'])
        u.sendStatusToChat(msg)


# In HARVEST_PATH, gunzip each *_radials.txt.gz and feed it to processRadialFile()
# Output lands in NCO_PATH
# ----------------------------------------------------------------------
async def processHarvest():
    fileList = sorted( HARVEST_PATH.glob('*') )
    for f in fileList:
        u.sendStatusToChat('aircraftTailRadarProcessor: Processing '+f.name)
        gunzipPath = u.gunzipFile(f)
        processRadialFile( str(gunzipPath), str(NCO_PATH),  \
                        numberOfGatesToAverage, maxNumberOfGates, \
                        flid, verbose )
        gunzipPath.unlink()
        u.moveFile(f, ARCHIVE_PATH, wrlog)


################
# Main
################
# ----------------------------------------------------------------------
# Check for command-line arguments
# ----------------------------------------------------------------------
#First check and make sure command-line argument requirements are met.
parser = argparse.ArgumentParser()
parser.add_argument("-v", action="store_true",
    help="prints version number and exits")
parser.add_argument("-f",
    help="Path of config file to use. Default = "+str(radarConfigPath))
args= parser.parse_args()

if args.v:
    print('\n aircraftTailRadarProcessor.py written by Remote Sensing Solutions')
    print(' June 2012   Version: 2.0')
    print(' April 2022  Adapted to Python 3 and exploiting AirGO repo utilities, by Sonia Otero, OMAO/MACC') 
    sys.exit()
    
if args.f:
    radarConfigPath = pathlib.Path(args.f)


# Get environment information
radarConfig = u.readINIConfig(radarConfigPath)

config = u.readPathConfig()
FLID_FILE= pathlib.Path( config.get('COMMON','FLID_FILE') )
flid,tailnum = u.getAircraftIds(FLID_FILE)
NCO_PATH = pathlib.Path( config.get('AIR','TDR_RADIALNCO') )

numberOfGatesToAverage = radarConfig.get('DEFAULT', 'numberOfGatesToAverage')
maxNumberOfGates = radarConfig.get('DEFAULT', 'maxNumberOfGates')
gateLength = radarConfig.get('DEFAULT', 'gateLength')
if radarConfig.has_section(tailnum):
    numberOfGatesToAverage = radarConfig.get(tailnum, 'numberOfGatesToAverage')
    maxNumberOfGates = radarConfig.get(tailnum, 'maxNumberOfGates')
    gateLength = radarConfig.get(tailnum, 'gateLength')

numberOfGatesToAverage = int(numberOfGatesToAverage)
maxNumberOfGates = int(maxNumberOfGates)
gateLength = int(gateLength)

print("length of gates is",gateLength,"m")
print("number of gates to average set to:")
print(numberOfGatesToAverage)
print("max number of gates to process set to:")
print(maxNumberOfGates)

INPUT_PATH = pathlib.Path( radarConfig.get('DEFAULT', 'inputFilePath') )
HARVEST_PATH = INPUT_PATH.joinpath('harvest')
ARCHIVE_PATH = NCO_PATH.joinpath('archive')

LOGFILE = pathlib.Path( config.get('AIR','TDR_LOG') )
wrlog = u.getLoggerFor(LOGFILE)
u.checkFileSize(LOGFILE, remove=True)

print('-> Running with the following parameters:')
print('   - Input File Path:   ',str(INPUT_PATH))
print('   - Output File Path:  ',str(NCO_PATH))
print('   - Archive File Path: ',str(ARCHIVE_PATH))
print('Running...')

u.guaranteePath(HARVEST_PATH)
u.guaranteePath(ARCHIVE_PATH)

# Create loop for asynchronous tasks
loop = asyncio.get_event_loop()
task1= loop.create_task( u.doPeriodicTask(5, scanIncomingPath) )
task2= loop.create_task( u.doPeriodicTask(20, processHarvest) )

try:
    loop.run_forever()
except KeyboardInterrupt:
    pass
finally:
    task1.cancel()
    task2.cancel()
    task3.cancel()
    loop.close()

sys.exit()